/* adept_optional.h -- Header for optionally applying automatic differentiation to an algorithm

    Copyright (C) 2012 Robin Hogan <r.j.hogan@reading.ac.uk>

    This file is part of the Adept library.

    This library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ADEPT_OPTIONAL_H
#define ADEPT_OPTIONAL_H 1

// This header file should be included by source files that will be
// compiled twice, the first time to produce the standard versions of
// a function or functions, and the second time with the compiler
// directive -DAUTOMATIC_DIFFERENTIATION, which compiles overloaded
// versions of these functions for use with automatic differentiation.

#ifdef AUTOMATIC_DIFFERENTIATION
// The compiler has been called with -DAUTOMATIC_DIFFERENTIATION:
// include the normal Adept header file.
#include "adept.h"

#else
// The compiler has *not* been called with -DAUTOMATIC_DIFFERENTIATION

namespace adept {
  // As in "adept.h", Real is a generic floating-point variable but
  // here we will treat it as double precision.
  typedef double Real;

  // If the user has labeled active types with aReal or adouble, treat
  // them as ordinary doubles.
  typedef Real aReal;
  typedef double adouble;

  // adept.h provides the function adept::value(adouble) that extracts
  // the underlying variable from an adouble argument. Since adouble
  // here is really just a double, we need to provide the same
  // function (in const and non-const versions) that essentially does
  // nothing but returns its argument.
  inline 
  double& value(double& arg) { return arg; }

  inline
  const double& value(const double& arg) { return arg; }
}

#endif // End of ifdef AUTOMATIC_DIFFERENTIATION

#endif
