#ifndef Timer_H
#define Timer_H 1

#include <sys/time.h>
#include <map>
#include <string>
#include <sstream>
#include <valarray>
#include <iostream>

class Timer {
public:
  Timer(int n_activities) : timings_(n_activities), 
			    current_activity_(-1), timer_on_(false) {
    last_time_.tv_sec = 0;
    last_time_.tv_usec = 0;
    timings_ = 0.0;
  }

  ~Timer() {
    double sum = 0.0;
    for (unsigned int i = 0; i < timings_.size(); i++) {
      std::cerr << "Time spent on activity " << i << ":\t" << timings_[i] << " s\n";
      sum += timings_[i];
    }
    std::cerr << "Total time: " << sum << " s\n";
  }

  void stop() {
    if (timer_on_) {
      timings_[current_activity_] += split_();
    }
    timer_on_ = false;
  };

  void start(int activity) {
    if (timer_on_) {
      timings_[current_activity_] += split_();
    }
    else {
      split_();
    }

    if (activity >= 0 && activity < timings_.size()) {
      current_activity_ = activity;
      timer_on_ = true;
    }
    else {
      // Activity out of range - to keep this inline function fast we
      // don't throw an exception but just don't record the time for
      // this event
      timer_on_ = false;
    }
  };

  const std::valarray<double>& timings() { return timings_; }

  std::string report(const char** activity_description) {
    std::stringstream str;
    for (int i = 0; i < timings_.size(); i++) {
      str << activity_description[i] << ": "
	  << timings_[i] << " s\n";
    }
    return str.str();
  }

private:
  double split_() {
    struct timeval time;
    gettimeofday(&time, NULL);
    double dsec = time.tv_sec - last_time_.tv_sec
      + 0.000001 * (time.tv_usec - last_time_.tv_usec);
    last_time_ = time;
    return dsec;
  }

  std::valarray<double> timings_;
  int current_activity_;
  timeval last_time_;
  bool timer_on_;
};

#endif
